<?php

declare(strict_types=1);

namespace Drupal\Tests\sel\Kernel;

use Drupal\Component\Utility\NestedArray;
use Drupal\Core\Entity\Display\EntityViewDisplayInterface;
use Drupal\Core\Field\FieldStorageDefinitionInterface;
use Drupal\field\Entity\FieldConfig;
use Drupal\field\Entity\FieldStorageConfig;
use Drupal\KernelTests\Core\Entity\EntityKernelTestBase;
use Drupal\link\LinkItemInterface;
use Drupal\Tests\sel\Traits\SelHostNameTrait;

/**
 * Tests the effect of SEL link field default configuration.
 *
 * @group sel
 *
 * @see \Drupal\sel\Plugin\Field\FieldFormatter\SelLinkFormatter
 */
class SelLinkFormatterDefaultsTest extends EntityKernelTestBase {

  use SelHostNameTrait;

  private const FIELD_NAME = 'test_page_link_field';

  /**
   * {@inheritdoc}
   */
  protected static $modules = [
    'entity_test',
    'link',
    'sel',
  ];

  /**
   * {@inheritdoc}
   */
  protected function setUp(): void {
    parent::setUp();

    $this->mockHostName();

    $this->installConfig(['system', 'entity_test', 'sel']);

    // Create a field with settings to validate.
    $field_storage = FieldStorageConfig::create([
      'field_name' => self::FIELD_NAME,
      'entity_type' => 'entity_test',
      'type' => 'link',
      'cardinality' => FieldStorageDefinitionInterface::CARDINALITY_UNLIMITED,
    ]);
    $field_storage->save();
    $field_config = FieldConfig::create([
      'field_storage' => $field_storage,
      'bundle' => 'entity_test',
      'settings' => [
        'title' => DRUPAL_OPTIONAL,
        'link_type' => LinkItemInterface::LINK_GENERIC,
      ],
    ]);
    $field_config->save();
  }

  /**
   * Tests how default setting config is applied on newly created link fields.
   *
   * @param array $linkFieldsDefaultConfig
   *   The configuration to save to the 'link_fields' key of SEL.
   * @param array $expectedDefaultConfig
   *   The expected default configuration at a newly created link field.
   *
   * @dataProvider providerTestLinkFormatterDefaults
   */
  public function testLinkFormatterDefaults(
    array $linkFieldsDefaultConfig = [],
    array $expectedDefaultConfig = [],
  ): void {
    $config = $this->config('sel.settings');
    $config->set('link_fields', array_merge($config->get('link_fields'), $linkFieldsDefaultConfig))->save();

    $entityViewDisplay = \Drupal::service('entity_display.repository')
      ->getViewDisplay('entity_test', 'entity_test', 'default');
    $this->assertInstanceOf(EntityViewDisplayInterface::class, $entityViewDisplay);
    $entityViewDisplay->setComponent(self::FIELD_NAME)->save();

    // Trim length, url rendering settings, general rel and target attribute
    // settings are out of our control.
    $expected = NestedArray::mergeDeep(
      $expectedDefaultConfig,
      [
        'settings' => [
          'trim_length' => 80,
          'url_only' => FALSE,
          'url_plain' => FALSE,
          'rel' => '',
          'target' => '',
        ],
      ],
    );

    $actualSettingsCleaned = array_diff_key(
      $entityViewDisplay->getComponent(self::FIELD_NAME),
      // Exclude label_display, third_party_settings, weight and region.
      [
        'label' => 1,
        'third_party_settings' => 1,
        'weight' => 1,
        'region' => 1,
      ],
    );
    // Also remove spamspan settings.
    unset($actualSettingsCleaned['settings']['spamspan']);

    $this->assertEquals($expected, $actualSettingsCleaned);
  }

  /**
   * Test data provider of ::testLinkFormatterDefaults.
   *
   * @return array[]
   *   The test cases.
   */
  public static function providerTestLinkFormatterDefaults(): array {
    return [
      'Disabled' => [
        'linkFieldsDefaultConfig' => [
          'default' => FALSE,
        ],
        'expectedDefaultConfig' => [
          'type' => 'link',
        ],
      ],
      'Enabled, default settings' => [
        'linkFieldsDefaultConfig' => [
          'default' => TRUE,
        ],
        'expectedDefaultConfig' => [
          'type' => 'sel_link',
          'settings' => [
            'sel' => [
              'auto_target' => TRUE,
              'target_attribute' => '_blank',
              'external_rel' => 'noreferrer',
              'external_rel_optionals' => [],
              'sanitize_email' => FALSE,
            ],
          ],
        ],
      ],
      'Enabled, custom target settings' => [
        'linkFieldsDefaultConfig' => [
          'default' => TRUE,
          'target_attribute_default' => 'foo_bar_baz',
        ],
        'expectedDefaultConfig' => [
          'type' => 'sel_link',
          'settings' => [
            'sel' => [
              'auto_target' => TRUE,
              'target_attribute' => 'foo_bar_baz',
              'external_rel' => 'noreferrer',
              'external_rel_optionals' => [],
              'sanitize_email' => FALSE,
            ],
          ],
        ],
      ],
    ];
  }

}
