<?php

namespace Drupal\Tests\sel\Unit;

use Drupal\Tests\sel\Traits\SelHostNameTrait;
use Drupal\sel\Plugin\Filter\SelLinkFilter;
use Drupal\Tests\sel\Traits\SelLinkTestCasesTrait;
use Drupal\Tests\UnitTestCase;

/**
 * Tests that links in formatted long text fields are well-handled.
 *
 * @coversDefaultClass \Drupal\sel\Plugin\Filter\SelLinkFilter
 *
 * @group sel
 */
class SelLinkFilterTest extends UnitTestCase {

  use SelHostNameTrait;
  use SelLinkTestCasesTrait {
    selLinkFilterDefaultTestCases as linkFilterDefaultDataProvider;
    selLinkFilterCustomizedTestCases as linkFilterCustomizedDataProvider;
  }

  /**
   * SelLinkFilter with default settings.
   *
   * @var \Drupal\sel\Plugin\Filter\SelLinkFilter
   */
  protected $filterDefault;

  /**
   * SelLinkFilter with customized settings.
   *
   * @var \Drupal\sel\Plugin\Filter\SelLinkFilter
   */
  protected $filterCustomized;

  /**
   * {@inheritdoc}
   */
  protected function setUp(): void {
    parent::setUp();

    $this->mockHostName();

    $config_default['settings'] = [
      'rel_required' => 'noreferrer',
      'rel_optionals' => [],
    ];

    $config_customized['settings'] = [
      'rel_required' => 'noopener',
      'rel_optionals' => ['nofollow', 'noreferrer'],
    ];

    $this->filterDefault = new SelLinkFilter($config_default, 'filter_sel', ['provider' => 'sel']);
    $this->filterCustomized = new SelLinkFilter($config_customized, 'filter_sel', ['provider' => 'sel']);
  }

  /**
   * Tests link filter with noreferrer and without optionals.
   *
   * @param string $html
   *   Input HTML.
   * @param string $expected
   *   The expected output string.
   *
   * @dataProvider linkFilterDefaultDataProvider
   */
  public function testSelLinkFilter($html, $expected): void {
    $this->assertSame($expected, (string) $this->filterDefault->process($html, 'en'));
  }

  /**
   * Tests link filter with noopener (required), nofollow and noreferrer.
   *
   * @param string $html
   *   Input HTML.
   * @param string $expected
   *   The expected output string.
   *
   * @dataProvider linkFilterCustomizedDataProvider
   */
  public function testCustomizedSelLinkFilter($html, $expected): void {
    $this->assertSame($expected, (string) $this->filterCustomized->process($html, 'en'));
  }

  /**
   * Tests target_attribute settings of the filter plugin.
   *
   * @param array $settings
   *   The settings of the filter plugin.
   * @param string $html
   *   The HTML to process.
   * @param string $expected
   *   The expected process HTML.
   *
   * @dataProvider providerTestTargetAttributeSettings
   */
  public function testTargetAttributeSettings(
    array $settings,
    string $html,
    string $expected,
  ) {
    $filter = new SelLinkFilter(
      ['settings' => $settings],
      'filter_sel',
      ['provider' => 'sel'],
    );
    $this->assertSame($expected, (string) $filter->process($html, 'en'));
  }

  /**
   * Test data provider of ::testTargetAttributeSettings.
   *
   * @return array[]
   *   The test cases.
   */
  public static function providerTestTargetAttributeSettings(): array {
    return [
      'Default settings' => [
        'settings' => [],
        'html' => '<a href="https://example.com">External link</a>',
        'expected' => '<a href="https://example.com" target="_blank" rel="noreferrer">External link</a>',
      ],
      'Custom target attribute config' => [
        'settings' => ['target_attribute' => 'foo_bar_baz'],
        'html' => '<a href="https://example.com">External link</a>',
        'expected' => '<a href="https://example.com" target="foo_bar_baz" rel="noreferrer">External link</a>',
      ],
      'Custom target attribute config, preexisting target attribute' => [
        'settings' => ['target_attribute' => 'foo_bar_baz'],
        'html' => '<a href="https://example.com" target="_blank" rel="noreferrer">External link with noreferrer attribute</a>',
        'expected' => '<a href="https://example.com" target="foo_bar_baz" rel="noreferrer">External link with noreferrer attribute</a>',
      ],
    ];
  }

}
